@@ -42,6 +42,7 @@
 
 ;;; Code:
 
+(eval-when-compile (require 'cl))
 (require 'w3m)
 
 (defconst w3m-weather-completion-table
@@ -104,7 +105,7 @@
 	       "toukyoutoizushotounanbu" "izushotounanbu")
 	      ("14/4610" "$B?@F`@n8)!&ElIt(B" "kanagawakentoubu")
 	      ("14/4620" "$B?@F`@n8)!&@>It(B" "kanagawakenseibu")
-	      ("15/5410" "$B?73c8)!&2<1[(B" "niigatakenshimoetsu" "shimoetsu")
+	      ("15/5410" "$B?73c8)!&2<1[(B" "niigatakenkaetsu" "kaetsu")
 	      ("15/5420" "$B?73c8)!&Cf1[(B" "niigatakenchuuetsu" "chuuetsu")
 	      ("15/5430" "$B?73c8)!&>e1[(B" "niigatakenjouetsu" "jouetsu")
 	      ("15/5440" "$B?73c8)!&:4EO(B" "niigatakensado" "sado")
@@ -169,7 +170,7 @@
 	      ("41/8510" "$B:42l8)!&FnIt(B" "sagakennanbu")
 	      ("41/8520" "$B:42l8)!&KLIt(B" "sagakenhokubu")
 	      ("42/700" "$BD9:j8)!&0m4tBPGO(B"
-	       "nagasakikeniktsushima" "iki" "tsushima")
+	       "nagasakikeniktsushima" "iki" "tsushima" "ikitsushima")
 	      ("42/800" "$BD9:j8)!&8^Eg(B" "nagasakikengotou" "gotou")
 	      ("42/8410" "$BD9:j8)!&FnIt(B" "nagasakikennanbu")
 	      ("42/8420" "$BD9:j8)!&KLIt(B" "nagasakikenhokubu")
@@ -184,8 +185,8 @@
 	      ("44/8340" "$BBgJ,8)!&FnIt(B" "ooitakennanbu")
 	      ("45/8710" "$B5\:j8)!&FnItJ?LnIt(B" "miyazakikennanbuheiyabu")
 	      ("45/8720" "$B5\:j8)!&KLItJ?LnIt(B" "miyazakikenhokubuheiyabu")
-	      ("45/8730" "$B5\:j8)!&FnIt;31h$$(B" "miyazakikennabuyamazoi")
-	      ("45/8740" "$B5\:j8)!&KLIt;31h$$(B" "miyazakikennanbuyamazoi")
+	      ("45/8730" "$B5\:j8)!&FnIt;31h$$(B" "miyazakikennanbuyamazoi")
+	      ("45/8740" "$B5\:j8)!&KLIt;31h$$(B" "miyazakikenhokubuyamazoi")
 	      ("46/8810" "$B</;yEg8)!&;'K`(B" "kagoshimakensatsuma" "satsuma")
 	      ("46/8820" "$B</;yEg8)!&Bg6y(B" "kagoshimakenoosumi" "oosumi")
 	      ("46/900" "$B</;yEg8)!&<o;REg!&205WEg(B"
@@ -225,7 +226,6 @@
 	   (hepburn-regexp
 	    (format "\\(\\`\\|[aiueo]\\)\\(n\\([^aiueoy]\\)\\|%s\\)"
 		    (regexp-opt (mapcar (function car) hepburn-table))))
-	   (hepburn-candidates)
 	   ;; $BD92;$NM-L5$K$h$kGI@87A$NI=(B
 	   (prolonged-table
 	    (let (table)
@@ -258,65 +258,63 @@
 	   ;; $BGI@87A$NI=$K>h$C$F$$$kJ8;zNs$rC5$9@55,I=8=(B
 	   (prolonged-regexp (format "\\(\\`\\|[aiueo]\\)\\(%s\\)"
 				     (regexp-opt (mapcar (function car)
-							 prolonged-table))))
-	   (prolonged-candidates)
-	   (romaji-candidates))
-      (setq hepburn-candidates
-	    (lambda (str)
-	      "$B%X%\%s<0$H71Na<0$N:9$K$h$C$F@8$8$kGI@87A$rF@$k(B"
-	      (if (string-match hepburn-regexp str)
-		  (let ((prefix (substring str 0 (match-beginning 2)))
-			(candidates (if (match-beginning 3)
-					'("n" "nn")
-				      (assoc (match-string 2 str)
-					     hepburn-table)))
-			(suffixes
-			 (funcall hepburn-candidates
-				  (substring str (or (match-beginning 3)
-						     (match-end 0)))))
-			(buf))
-		    (dolist (x candidates)
-		      (dolist (y suffixes)
-			(push (concat prefix x y) buf)))
-		    buf)
-		(list str)))
-	    prolonged-candidates
-	    (lambda (str)
-	      "$BD92;$NM-L5$K$h$C$F@8$8$kGI@87A$rF@$k(B"
-	      (let (buf)
-		(if (string-match prolonged-regexp str)
+							 prolonged-table)))))
+      (labels ((hepburn-candidates
+		(str)
+		"$B%X%\%s<0$H71Na<0$N:9$K$h$C$F@8$8$kGI@87A$rF@$k(B"
+		(if (string-match hepburn-regexp str)
 		    (let ((prefix (substring str 0 (match-beginning 2)))
-			  (candidates (assoc (match-string 2 str)
-					     prolonged-table))
-			  (suffixes (funcall prolonged-candidates
-					     (substring str (match-end 0)))))
+			  (candidates (if (match-beginning 3)
+					  '("n" "nn")
+					(assoc (match-string 2 str)
+					       hepburn-table)))
+			  (suffixes
+			   (hepburn-candidates
+			    (substring str (or (match-beginning 3)
+					       (match-end 0)))))
+			  (buf))
 		      (dolist (x candidates)
 			(dolist (y suffixes)
-			  (push (concat prefix x y) buf))))
-		  (setq buf (list str)))
-		(dolist (x buf)
-		  (when (string-match "\\(\\`\\|[aiue]\\)oo" x)
-		    (let ((prefix (substring x 0 (match-end 1)))
-			  (suffix (substring x (match-end 0))))
-		      (dolist (y '("o" "oh" "o-"))
-			(push (concat prefix y suffix) buf)))))
-		buf))
-	    romaji-candidates
-	    (lambda (str)
-	      "$BA4$F$NGI@87A$rF@$k(B"
-	      (let (buf)
-		(dolist (x (funcall hepburn-candidates str))
-		  (dolist (y (funcall prolonged-candidates x))
-		    (push y buf)))
-		buf)))
-      (dolist (area alist)
-	(let ((url (format format (car area)))
-	      (kanji (cadr area)))
-	  (push (list kanji (nth 2 area) url) table)
-	  (dolist (romaji (cddr area))
-	    (dolist (x (funcall romaji-candidates romaji))
-	      (push (list x kanji) table)))))
-      (nreverse table)))
+			  (push (concat prefix x y) buf)))
+		      buf)
+		  (list str)))
+	       (prolonged-candidates
+		(str)
+		"$BD92;$NM-L5$K$h$C$F@8$8$kGI@87A$rF@$k(B"
+		(let (buf)
+		  (if (string-match prolonged-regexp str)
+		      (let ((prefix (substring str 0 (match-beginning 2)))
+			    (candidates (assoc (match-string 2 str)
+					       prolonged-table))
+			    (suffixes (prolonged-candidates
+				       (substring str (match-end 0)))))
+			(dolist (x candidates)
+			  (dolist (y suffixes)
+			    (push (concat prefix x y) buf))))
+		    (setq buf (list str)))
+		  (dolist (x buf)
+		    (when (string-match "\\(\\`\\|[aiue]\\)oo" x)
+		      (let ((prefix (substring x 0 (match-end 1)))
+			    (suffix (substring x (match-end 0))))
+			(dolist (y '("o" "oh" "o-"))
+			  (push (concat prefix y suffix) buf)))))
+		  buf))
+	       (romaji-candidates
+		(str)
+		"$BA4$F$NGI@87A$rF@$k(B"
+		(let (buf)
+		  (dolist (x (hepburn-candidates str))
+		    (dolist (y (prolonged-candidates x))
+		      (push y buf)))
+		  buf)))
+	(dolist (area alist)
+	  (let ((url (format format (car area)))
+		(kanji (cadr area)))
+	    (push (list kanji (nth 2 area) url) table)
+	    (dolist (romaji (cddr area))
+	      (dolist (x (romaji-candidates romaji))
+		(push (list x kanji) table)))))
+	(nreverse table))))
   "Completion table of areas and urls.")
 
 (defcustom w3m-weather-default-area
